use auth::AuthGuard;
use util::Address;

pub mod util;
pub mod auth;

pub enum DataStorageError {
    AlreadyExists,
    AntiSpam(String)
}

pub trait DataStorageItem {
    type Id;
    type Address: Address;
    fn id(&self) -> Self::Id;
    fn address_info(&self) -> Self::Address;
}

pub trait DataStorage<Auth: AuthGuard, Item: DataStorageItem>: IntoIterator<Item = Item> {
    fn update_instance(&mut self, address: &impl Address, guard: Option<Auth>) 
        -> Result<Item::Id, DataStorageError>;

    fn send_heartbeat(&mut self, address: &impl Address) 
        -> Result<(),DataStorageError>;

    fn peek_instance(&self, id: Item::Id);
}

